import express from 'express';
import _ from 'lodash';
import ServiceManager from '../ServiceManager';

class App {
  constructor() {
    const sm = ServiceManager.getInstance();
    sm.setService('Express', express());
    this.serviceManger = sm;
  }

  get express() {
    return this.serviceManger.getService('Express');
  }

  set config(c) {
    this.serviceManger.setService('Config', c);
  }

  get config() {
    return this.serviceManger.getService('Config');
  }

  async _init(config) {
    this.serviceManger
      .setService('App', App.getInstance())
      .setService('Config', config);

    await this._middleware();
  }

  _middleware() {
    return new Promise(() => {
      this._defaultMiddleware();
    });
  }

  _defaultMiddleware() {
    if (Array.isArray(this.config.middleware)) {
      _.forEach(this.config.middleware, item => {
        this.express.use(item);
      });
    }
  }

  _listen(port) {
    this.express.listen(port);
  }

  _finish() {
    
  }

  static getInstance() {
    if (!App._singleton) {
      App._singleton = new App();
    }
    return App._singleton;
  }

  run(config) {
    this._init(config);
    this._listen(config.app.port);
    this._finish();
  }
}
export default App;
